@@ -8,7 +8,7 @@ from ..core.data import ImageInterface
 from ..core import Dimension, Element2D, Overlay, Dataset
 from ..core.boundingregion import BoundingRegion, BoundingBox
 from ..core.sheetcoords import SheetCoordinateSystem, Slice
-from ..core.util import max_range
+from ..core.util import max_range, dimension_range, compute_density, datetime_types
 from .chart import Curve
 from .tabular import Table
 from .util import compute_edges, compute_slice_bounds, categorical_aggregate2d
@@ -39,11 +39,11 @@ class Raster(Element2D):
                        bounds=(1, 1), doc="""
         The dimension description of the data held in the matrix.""")
 
-    def __init__(self, data, extents=None, **params):
+    def __init__(self, data, kdims=None, vdims=None, extents=None, **params):
         if extents is None:
             (d1, d2) = data.shape[:2]
             extents = (0, 0, d2, d1)
-        super(Raster, self).__init__(data, extents=extents, **params)
+        super(Raster, self).__init__(data, kdims=kdims, vdims=vdims, extents=extents, **params)
 
 
     def __getitem__(self, slices):
@@ -71,14 +71,8 @@ class Raster(Element2D):
         idx = self.get_dimension_index(dim)
         if data_range and idx == 2:
             dimension = self.get_dimension(dim)
-            drange = self.data.min(), self.data.max()
-            drange = max_range([drange, dimension.soft_range])
-            if dimension.range[0] is not None:
-                return (dimension.range[0], drange[1])
-            elif dimension.range[1] is not None:
-                return (drange[0], dimension.range[1])
-            else:
-                return drange
+            lower, upper = np.nanmin(self.data), np.nanmax(self.data)
+            return dimension_range(lower, upper, dimension)
         return super(Raster, self).range(dim, data_range)
 
 
@@ -233,17 +227,18 @@ class Image(Dataset, Raster, SheetCoordinateSystem):
                        bounds=(1, 1), doc="""
         The dimension description of the data held in the matrix.""")
 
-    def __init__(self, data, bounds=None, extents=None, xdensity=None, ydensity=None, **params):
+    def __init__(self, data, kdims=None, vdims=None, bounds=None, extents=None,
+                 xdensity=None, ydensity=None, **params):
         extents = extents if extents else (None, None, None, None)
         if data is None: data = np.array([[0]])
-        Dataset.__init__(self, data, extents=extents, **params)
+        Dataset.__init__(self, data, kdims=kdims, vdims=vdims, extents=extents, **params)
 
         dim2, dim1 = self.interface.shape(self, gridded=True)[:2]
         if bounds is None:
             xvals = self.dimension_values(0, False)
-            l, r, xdensity, _ = util.bound_range(xvals, xdensity)
+            l, r, xdensity, _ = util.bound_range(xvals, xdensity, self._time_unit)
             yvals = self.dimension_values(1, False)
-            b, t, ydensity, _ = util.bound_range(yvals, ydensity)
+            b, t, ydensity, _ = util.bound_range(yvals, ydensity, self._time_unit)
             bounds = BoundingBox(points=((l, b), (r, t)))
         elif np.isscalar(bounds):
             bounds = BoundingBox(radius=bounds)
@@ -252,8 +247,8 @@ class Image(Dataset, Raster, SheetCoordinateSystem):
             bounds = BoundingBox(points=((l, b), (r, t)))
 
         l, b, r, t = bounds.lbrt()
-        xdensity = xdensity if xdensity else dim1/float(r-l)
-        ydensity = ydensity if ydensity else dim2/float(t-b)
+        xdensity = xdensity if xdensity else compute_density(l, r, dim1, self._time_unit)
+        ydensity = ydensity if ydensity else compute_density(b, t, dim2, self._time_unit)
         SheetCoordinateSystem.__init__(self, bounds, xdensity, ydensity)
 
         if len(self.shape) == 3:
@@ -321,7 +316,9 @@ class Image(Dataset, Raster, SheetCoordinateSystem):
         if not sliced:
             if np.isscalar(data):
                 return data
-            return self.clone(data[self.ndims:], kdims=[], new_type=Dataset,
+            elif isinstance(data, tuple):
+                data = data[self.ndims:]
+            return self.clone(data, kdims=[], new_type=Dataset,
                               datatype=datatype)
         else:
             return self.clone(data, xdensity=self.xdensity, datatype=datatype,
@@ -367,6 +364,14 @@ class Image(Dataset, Raster, SheetCoordinateSystem):
         xs, ys = zip(*samples)
         yidx, xidx = self.sheet2matrixidx(np.array(xs), np.array(ys))
         yidx = shape[0]-yidx-1
+
+        # Detect out-of-bounds indices
+        out_of_bounds= (yidx<0) | (xidx<0) | (yidx>=shape[0]) | (xidx>=shape[1])
+        if out_of_bounds.any():
+            coords = [samples[idx] for idx in np.where(out_of_bounds)[0]]
+            raise IndexError('Coordinate(s) %s out of bounds for %s with bounds %s' %
+                             (coords, type(self).__name__, self.bounds.lbrt()))
+
         data = self.interface.ndloc(self, (yidx, xidx))
         return self.clone(data, new_type=Table, datatype=['dataframe', 'dict'])
 
@@ -413,10 +418,16 @@ class Image(Dataset, Raster, SheetCoordinateSystem):
         dimension = self.get_dimension(dim)
         low, high = super(Image, self).range(dim, data_range)
         if idx in [0, 1] and data_range and dimension.range == (None, None):
+            if self.interface.datatype == 'image':
+                l, b, r, t = self.bounds.lbrt()
+                return (b, t) if idx else (l, r)
             density = self.ydensity if idx else self.xdensity
             halfd = (1./density)/2.
+            if isinstance(low, datetime_types):
+                halfd = np.timedelta64(int(round(halfd)), self._time_unit)
             return (low-halfd, high+halfd)
-        return low, high
+        else:
+            return super(Image, self).range(dim, data_range)
 
 
     def table(self, datatype=None):
@@ -521,7 +532,7 @@ class RGB(Image):
         return rgb
 
 
-    def __init__(self, data, **params):
+    def __init__(self, data, kdims=None, vdims=None, **params):
         if isinstance(data, Overlay):
             images = data.values()
             if not all(isinstance(im, Image) for im in images):
@@ -534,12 +545,14 @@ class RGB(Image):
                 raise ValueError("Ranges must be defined on all the value dimensions of all the Images")
             arrays = [(im.data - r[0]) / (r[1] - r[0]) for r,im in zip(ranges, images)]
             data = np.dstack(arrays)
-        vdims = list(params.get('vdims', self.vdims))
+        if vdims is None:
+            vdims = list(self.vdims)
+        else:
+            vdims = list(vdims) if isinstance(vdims, list) else [vdims]
         if isinstance(data, np.ndarray):
             if data.shape[-1] == 4 and len(vdims) == 3:
                 vdims.append(self.alpha_dimension)
-                params['vdims'] = vdims
-        super(RGB, self).__init__(data, **params)
+        super(RGB, self).__init__(data, kdims=kdims, vdims=vdims, **params)
 
 
 
@@ -602,13 +615,14 @@ class QuadMesh(Raster):
 
     group = param.String(default="QuadMesh", constant=True)
 
-    kdims = param.List(default=[Dimension('x'), Dimension('y')])
+    kdims = param.List(default=[Dimension('x'), Dimension('y')],
+                       bounds=(2, 2), constant=True)
 
     vdims = param.List(default=[Dimension('z')], bounds=(1,1))
 
-    def __init__(self, data, **params):
+    def __init__(self, data, kdims=None, vdims=None, **params):
         data = self._process_data(data)
-        Element2D.__init__(self, data, **params)
+        Element2D.__init__(self, data, kdims=kdims, vdims=vdims, **params)
         self.data = self._validate_data(self.data)
         self._grid = self.data[0].ndim == 1
 
@@ -720,15 +734,14 @@ class QuadMesh(Raster):
                      for i in [1, 0])
 
 
-    def range(self, dimension):
+    def range(self, dimension, data_range=True):
         idx = self.get_dimension_index(dimension)
-        if idx in [0, 1]:
+        dim = self.get_dimension(dimension)
+        if idx in [0, 1, 2] and data_range:
             data = self.data[idx]
-            return np.min(data), np.max(data)
-        elif idx == 2:
-            data = self.data[idx]
-            return np.nanmin(data), np.nanmax(data)
-        super(QuadMesh, self).range(dimension)
+            lower, upper = np.nanmin(data), np.nanmax(data)
+            return dimension_range(lower, upper, dim)
+        return super(QuadMesh, self).range(dimension, data_range)
 
 
     def dimension_values(self, dimension, expanded=True, flat=True):
@@ -765,12 +778,13 @@ class HeatMap(Dataset, Element2D):
 
     group = param.String(default='HeatMap', constant=True)
 
-    kdims = param.List(default=[Dimension('x'), Dimension('y')])
+    kdims = param.List(default=[Dimension('x'), Dimension('y')],
+                       bounds=(2, 2), constant=True)
 
-    vdims = param.List(default=[Dimension('z')])
+    vdims = param.List(default=[Dimension('z')], constant=True)
 
-    def __init__(self, data, **params):
-        super(HeatMap, self).__init__(data, **params)
+    def __init__(self, data, kdims=None, vdims=None, **params):
+        super(HeatMap, self).__init__(data, kdims=kdims, vdims=vdims, **params)
         self.gridded = categorical_aggregate2d(self)
 
     @property
