@@ -5,9 +5,9 @@ import param
 import numpy as np
 from ...core import Dataset
 from ...element import ItemTable
+from ...streams import Buffer
 from ..plot import GenericElementPlot
 from .plot import BokehPlot
-from .util import bokeh_version
 
 
 class TablePlot(BokehPlot, GenericElementPlot):
@@ -17,14 +17,14 @@ class TablePlot(BokehPlot, GenericElementPlot):
     width = param.Number(default=400)
 
     style_opts = ['row_headers', 'selectable', 'editable',
-                  'sortable', 'fit_columns', 'width', 'height']
+                  'sortable', 'fit_columns']
 
     finalize_hooks = param.HookList(default=[], doc="""
         Optional list of hooks called when finalizing a column.
         The hook is passed the plot object and the displayed
         object, and other plotting handles can be accessed via plot.handles.""")
 
-    _update_handles = ['source', 'glyph']
+    _stream_data = True
 
     def __init__(self, element, plot=None, **params):
         super(TablePlot, self).__init__(element, **params)
@@ -32,6 +32,8 @@ class TablePlot(BokehPlot, GenericElementPlot):
         element_ids = self.hmap.traverse(lambda x: id(x), [Dataset, ItemTable])
         self.static = len(set(element_ids)) == 1 and len(self.keys) == len(self.hmap)
         self.callbacks = [] # Callback support on tables not implemented
+        dfstream = [s for s in self.streams if isinstance(s, Buffer)]
+        self.streaming = dfstream[0] if any(dfstream) else None
 
 
     def _execute_hooks(self, element):
@@ -45,14 +47,13 @@ class TablePlot(BokehPlot, GenericElementPlot):
                 self.warning("Plotting hook %r could not be applied:\n\n %s" % (hook, e))
 
 
-    def get_data(self, element, ranges=None, empty=False):
+    def get_data(self, element, ranges, style):
         dims = element.dimensions()
-        data = {d: np.array([]) if empty else element.dimension_values(d)
-                 for d in dims}
         mapping = {d.name: d.name for d in dims}
+        data = {d: element.dimension_values(d) for d in dims}
         data = {d.name: values if values.dtype.kind in "if" else list(map(d.pprint_value, values))
                 for d, values in data.items()}
-        return data, mapping
+        return data, mapping, style
 
 
     def initialize_plot(self, ranges=None, plot=None, plots=None, source=None):
@@ -65,18 +66,17 @@ class TablePlot(BokehPlot, GenericElementPlot):
         self.current_frame = element
         self.current_key = key
 
-        data, _ = self.get_data(element, ranges)
+        style = self.lookup_options(element, 'style')[self.cyclic_index]
+        data, _, style = self.get_data(element, ranges, style)
         if source is None:
             source = self._init_datasource(data)
         self.handles['source'] = source
 
         dims = element.dimensions()
         columns = [TableColumn(field=d.name, title=d.pprint_label) for d in dims]
-        properties = self.lookup_options(element, 'style')[self.cyclic_index]
-        if bokeh_version > '0.12.7':
-            properties['reorderable'] = False
+        style['reorderable'] = False
         table = DataTable(source=source, columns=columns, height=self.height,
-                          width=self.width, **properties)
+                          width=self.width, **style)
         self.handles['plot'] = table
         self.handles['glyph_renderer'] = table
         self._execute_hooks(element)
@@ -85,38 +85,23 @@ class TablePlot(BokehPlot, GenericElementPlot):
         return table
 
 
-    @property
-    def current_handles(self):
-        """
-        Returns a list of the plot objects to update.
-        """
-        handles = []
-        if self.static and not self.dynamic:
-            return handles
-
-
-        element = self.current_frame
-        previous_id = self.handles.get('previous_id', None)
-        current_id = None if self.current_frame is None else element._plot_id
-        for handle in self._update_handles:
-            if (handle == 'source' and self.dynamic and current_id == previous_id):
-                continue
-            if handle in self.handles:
-                handles.append(self.handles[handle])
-
-        # Cache frame object id to skip updating if unchanged
-        if self.dynamic:
-            self.handles['previous_id'] = current_id
-
-        return handles
-
-
     def update_frame(self, key, ranges=None, plot=None):
         """
         Updates an existing plot with data corresponding
         to the key.
         """
         element = self._get_frame(key)
+
+        # Cache frame object id to skip updating data if unchanged
+        previous_id = self.handles.get('previous_id', None)
+        current_id = element._plot_id
+        self.handles['previous_id'] = current_id
+        self.static_source = (self.dynamic and (current_id == previous_id))
+        if (element is None or (not self.dynamic and self.static) or
+            (self.streaming and self.streaming.data is self.current_frame.data
+             and not self.streaming._triggering) or self.static_source):
+            return
         source = self.handles['source']
-        data, _ = self.get_data(element, ranges)
+        style = self.lookup_options(element, 'style')[self.cyclic_index]
+        data, _, style = self.get_data(element, ranges, style)
         self._update_datasource(source, data)
